@@ -1,6 +1,11 @@
 #ifndef COMMON_H
 #define COMMON_H
 
+#ifdef __FreeBSD__
+#define stat64 stat
+#define lstat64 lstat
+#endif
+
 #include <stdbool.h>
 
 #define BLOCKING_TIMEOUT -1
