@@ -34,9 +34,10 @@ extern int optind, opterr, optopt;
 // METHODS
 bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
                 long int *timeout, int *recursive, bool *csv, bool *daemon,
-                bool *syslog, char **format, char **timefmt, char **fromfile,
-                char **outfile, char **exc_regex, char **exc_iregex,
-                char **inc_regex, char **inc_iregex);
+                bool *syslog, bool *no_dereference, char **format,
+                char **timefmt, char **fromfile, char **outfile,
+                char **exc_regex, char **exc_iregex, char **inc_regex,
+                char **inc_iregex, bool *no_newline);
 
 void print_help();
 
@@ -136,6 +137,7 @@ int main(int argc, char **argv) {
     bool csv = false;
     bool dodaemon = false;
     bool syslog = false;
+    bool no_dereference = false;
     char *format = NULL;
     char *timefmt = NULL;
     char *fromfile = NULL;
@@ -144,13 +146,14 @@ int main(int argc, char **argv) {
     char *exc_iregex = NULL;
     char *inc_regex = NULL;
     char *inc_iregex = NULL;
+    bool no_newline = false;
     int fd;
 
     // Parse commandline options, aborting if something goes wrong
     if (!parse_opts(&argc, &argv, &events, &monitor, &quiet, &timeout,
-                    &recursive, &csv, &dodaemon, &syslog, &format, &timefmt,
-                    &fromfile, &outfile, &exc_regex, &exc_iregex, &inc_regex,
-                    &inc_iregex)) {
+                    &recursive, &csv, &dodaemon, &syslog, &no_dereference,
+                    &format, &timefmt, &fromfile, &outfile, &exc_regex,
+                    &exc_iregex, &inc_regex, &inc_iregex, &no_newline)) {
         return EXIT_FAILURE;
     }
 
@@ -190,6 +193,9 @@ int main(int argc, char **argv) {
     if (monitor && recursive) {
         events = events | IN_CREATE | IN_MOVED_TO | IN_MOVED_FROM;
     }
+    if (no_dereference) {
+        events = events | IN_DONT_FOLLOW;
+    }
 
     FileList list = construct_path_list(argc, argv, fromfile);
 
@@ -412,9 +418,10 @@ int main(int argc, char **argv) {
 
 bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
                 long int *timeout, int *recursive, bool *csv, bool *daemon,
-                bool *syslog, char **format, char **timefmt, char **fromfile,
-                char **outfile, char **exc_regex, char **exc_iregex,
-                char **inc_regex, char **inc_iregex) {
+                bool *syslog, bool *no_dereference, char **format,
+                char **timefmt, char **fromfile, char **outfile,
+                char **exc_regex, char **exc_iregex, char **inc_regex,
+                char **inc_iregex, bool *no_newline) {
     assert(argc);
     assert(argv);
     assert(events);
@@ -424,6 +431,7 @@ bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
     assert(csv);
     assert(daemon);
     assert(syslog);
+    assert(no_dereference);
     assert(format);
     assert(timefmt);
     assert(fromfile);
@@ -445,11 +453,11 @@ bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
     const char *regex_warning =
         "only the last option will be taken into consideration.\n";
 
-    // format with trailing newline
-    static char *newlineformat;
+    // format provided by the user
+    static char *customformat = NULL;
 
     // Short options
-    static const char opt_string[] = "mrhcdsqt:fo:e:";
+    static const char opt_string[] = "mrhcdsPqt:fo:e:";
 
     // Long options
     static const struct option long_opts[] = {
@@ -463,7 +471,9 @@ bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
         {"csv", no_argument, NULL, 'c'},
         {"daemon", no_argument, NULL, 'd'},
         {"syslog", no_argument, NULL, 's'},
+        {"no-dereference", no_argument, NULL, 'P'},
         {"format", required_argument, NULL, 'n'},
+        {"no-newline", no_argument, NULL, '0'},
         {"timefmt", required_argument, NULL, 'i'},
         {"fromfile", required_argument, NULL, 'z'},
         {"outfile", required_argument, NULL, 'o'},
@@ -519,6 +529,11 @@ bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
             (*syslog) = true;
             break;
 
+        // --no-dereference or -P
+        case 'P':
+            (*no_dereference) = true;
+            break;
+
         // --filename or -f
         case 'f':
             fprintf(stderr, "The '--filename' option no longer exists.  "
@@ -529,10 +544,13 @@ bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
 
         // --format
         case 'n':
-            newlineformat = (char *)malloc(strlen(optarg) + 2);
-            strcpy(newlineformat, optarg);
-            strcat(newlineformat, "\n");
-            (*format) = newlineformat;
+            customformat = (char *)malloc(strlen(optarg) + 2);
+            strcpy(customformat, optarg);
+            break;
+
+        // --no-newline
+        case '0':
+            (*no_newline) = true;
             break;
 
         // --timefmt
@@ -610,6 +628,13 @@ bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
         curr_opt = getopt_long(*argc, *argv, opt_string, long_opts, NULL);
     }
 
+    if (customformat) {
+        if(!(*no_newline)) {
+            strcat(customformat, "\n");
+        }
+        (*format) = customformat;
+    }
+
     if (*exc_regex && *exc_iregex) {
         fprintf(stderr, "--exclude and --excludei cannot both be specified.\n");
         return false;
@@ -632,6 +657,11 @@ bool parse_opts(int *argc, char ***argv, int *events, bool *monitor, int *quiet,
         return false;
     }
 
+    if (!*format && *no_newline) {
+        fprintf(stderr, "--no-newline cannot be specified without --format.\n");
+        return false;
+    }
+
     if (!*format && *timefmt) {
         fprintf(stderr, "--timefmt cannot be specified without --format.\n");
         return false;
@@ -693,6 +723,8 @@ void print_help() {
         "\t-d|--daemon   \tSame as --monitor, except run in the background\n"
         "\t              \tlogging events to a file specified by --outfile.\n"
         "\t              \tImplies --syslog.\n");
+    printf("\t-P|--no-dereference\n"
+           "\t              \tDo not follow symlinks.\n");
     printf("\t-r|--recursive\tWatch directories recursively.\n");
     printf("\t--fromfile <file>\n"
            "\t              \tRead files to watch from <file> or `-' for "
@@ -704,6 +736,8 @@ void print_help() {
     printf("\t-qq           \tPrint nothing (not even events).\n");
     printf("\t--format <fmt>\tPrint using a specified printf-like format\n"
            "\t              \tstring; read the man page for more details.\n");
+    printf("\t--no-newline  \tDon't print newline symbol after\n"
+           "\t              \t--format string.\n");
     printf("\t--timefmt <fmt>\tstrftime-compatible format string for use with\n"
            "\t              \t%%T in --format string.\n");
     printf("\t-c|--csv      \tPrint events in CSV format.\n");
