@@ -1,4 +1,3 @@
-// FIXME this is cheating!  Make this use only the public API.
 #include "../config.h"
 #include "../libinotifytools/src/inotifytools_p.h"
 #include "common.h"
@@ -31,8 +30,8 @@ extern int optind, opterr, optopt;
 // METHODS
 bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
                 int *verbose, int *zero, int *sort, int *recursive,
-                char **fromfile, char **exc_regex, char **exc_iregex,
-                char **inc_regex, char **inc_iregex);
+                int *no_dereference, char **fromfile, char **exc_regex,
+                char **exc_iregex, char **inc_regex, char **inc_iregex);
 
 void print_help();
 
@@ -74,6 +73,7 @@ int main(int argc, char **argv) {
     int verbose = 0;
     zero = 0;
     int recursive = 0;
+    int no_dereference = 0;
     char *fromfile = 0;
     sort = -1;
     done = false;
@@ -86,8 +86,8 @@ int main(int argc, char **argv) {
 
     // Parse commandline options, aborting if something goes wrong
     if (!parse_opts(&argc, &argv, &events, &timeout, &verbose, &zero, &sort,
-                    &recursive, &fromfile, &exc_regex, &exc_iregex, &inc_regex,
-                    &inc_iregex)) {
+                    &recursive, &no_dereference, &fromfile, &exc_regex,
+                    &exc_iregex, &inc_regex, &inc_iregex)) {
         return EXIT_FAILURE;
     }
 
@@ -119,6 +119,8 @@ int main(int argc, char **argv) {
     // If events is still 0, make it all events.
     if (!events)
         events = IN_ALL_EVENTS;
+    if (no_dereference)
+        events = events | IN_DONT_FOLLOW;
 
     FileList list = construct_path_list(argc, argv, fromfile);
 
@@ -362,18 +364,19 @@ int print_info() {
     return EXIT_SUCCESS;
 }
 
-bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
-                int *verbose, int *zero, int *sort, int *recursive,
-                char **fromfile, char **exc_regex, char **exc_iregex,
-                char **inc_regex, char **inc_iregex) {
+bool parse_opts(int *argc, char ***argv, int *e, long int *timeout,
+                int *verbose, int *z, int *s, int *recursive,
+                int *no_dereference, char **fromfile, char **exc_regex,
+                char **exc_iregex, char **inc_regex, char **inc_iregex) {
     assert(argc);
     assert(argv);
-    assert(events);
+    assert(e);
     assert(timeout);
     assert(verbose);
-    assert(zero);
-    assert(sort);
+    assert(z);
+    assert(s);
     assert(recursive);
+    assert(no_dereference);
     assert(fromfile);
     assert(exc_regex);
     assert(exc_iregex);
@@ -385,7 +388,7 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
     bool sort_set = false;
 
     // Short options
-    static const char opt_string[] = "hra:d:zve:t:";
+    static const char opt_string[] = "hrPa:d:zve:t:";
 
     // Construct array
     static const struct option long_opts[] = {
@@ -397,6 +400,7 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
         {"ascending", required_argument, NULL, 'a'},
         {"descending", required_argument, NULL, 'd'},
         {"recursive", no_argument, NULL, 'r'},
+        {"no-dereference", no_argument, NULL, 'P'},
         {"fromfile", required_argument, NULL, 'o'},
         {"exclude", required_argument, NULL, 'c'},
         {"excludei", required_argument, NULL, 'b'},
@@ -427,10 +431,13 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
         case 'r':
             ++(*recursive);
             break;
+        case 'P':
+            ++(*no_dereference);
+            break;
 
         // --zero or -z
         case 'z':
-            ++(*zero);
+            ++(*z);
             break;
 
         // --exclude
@@ -484,7 +491,7 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
             }
 
             // Add the new event to the event mask
-            (*events) = ((*events) | new_event);
+            (*e) = ((*e) | new_event);
 
             break;
 
@@ -495,7 +502,7 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
                 return false;
             }
             if (0 == strcasecmp(optarg, "total")) {
-                (*sort) = 0;
+                (*s) = 0;
             } else if (0 == strcasecmp(optarg, "move")) {
                 fprintf(stderr, "Cannot sort by `move' event; please use "
                                 "`moved_from' or `moved_to'.\n");
@@ -515,7 +522,7 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
                     return false;
                 }
 
-                (*sort) = event;
+                (*s) = event;
             }
             sort_set = true;
             break;
@@ -527,7 +534,7 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
                 return false;
             }
             if (0 == strcasecmp(optarg, "total")) {
-                (*sort) = -1;
+                (*s) = -1;
             } else {
                 int event = inotifytools_str_to_event(optarg);
 
@@ -539,7 +546,7 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
                     return false;
                 }
 
-                (*sort) = -event;
+                (*s) = -event;
             }
             break;
         }
@@ -550,8 +557,8 @@ bool parse_opts(int *argc, char ***argv, int *events, long int *timeout,
     (*argc) -= optind;
     *argv = &(*argv)[optind];
 
-    if ((*sort) != 0 && (*sort) != -1 &&
-        !(abs(*sort) & ((*events) ? (*events) : IN_ALL_EVENTS))) {
+    if ((*s) != 0 && (*s) != -1 &&
+        !(abs(*s) & ((*e) ? (*e) : IN_ALL_EVENTS))) {
         fprintf(stderr, "Can't sort by an event which isn't being watched "
                         "for!\n");
         return false;
@@ -603,6 +610,8 @@ void print_help() {
            "\t\tif they consist only of zeros (the default is to not output\n"
            "\t\tthese rows and columns).\n");
     printf("\t-r|--recursive\tWatch directories recursively.\n");
+    printf("\t-P|--no-dereference\n"
+           "\t\tDo not follow symlinks.\n");
     printf("\t-t|--timeout <seconds>\n"
            "\t\tListen only for specified amount of time in seconds; if\n"
            "\t\tomitted or negative, inotifywatch will execute until receiving "
